#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightColor;

uniform vec3 camera_world;

uniform vec4 materialCoefficients; // x = ambient, y = diffuse, z = specular, w=reflectiveness
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D secondTexture;
uniform samplerCube skybox;
uniform int mode;
uniform float userDefinedBrightness;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct bluePointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} bluePointL;

uniform struct redPointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} redPointL;

uniform struct normalPointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} normalPointL;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
if(mode==3){

    vec3 n = texture(secondTexture, vert.uv).rgb;
    n = normalize(n * 2.0 - 1.0);  // this normal is in tangent space
   
    vec3 v = normalize(camera_world - vert.position_world);
	vec3 r = reflect(-v,n);
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb *(1.0-materialCoefficients.w);
	texColor+=texture(skybox,r).rgb * materialCoefficients.w;
	vec4 result = vec4(texColor * materialCoefficients.x, 1) + + vec4(userDefinedBrightness,userDefinedBrightness,userDefinedBrightness,1.0); // ambient
	
			
	// add point light contribution
	result.rgb += phong(n, redPointL.position - vert.position_world, v, redPointL.color * texColor, materialCoefficients.y, redPointL.color, materialCoefficients.z, specularAlpha, true, redPointL.attenuation);
	result.rgb += phong(n, bluePointL.position - vert.position_world, v, bluePointL.color * texColor, materialCoefficients.y, bluePointL.color, materialCoefficients.z, specularAlpha, true, bluePointL.attenuation);
	result.rgb += phong(n, normalPointL.position - vert.position_world, v, normalPointL.color * texColor, materialCoefficients.y, normalPointL.color, materialCoefficients.z, specularAlpha, true, normalPointL.attenuation);
	
		color=vec4(result.rgb,1.0) ;
		brightColor = vec4(0.0,0.0,0.0,1.0);

}
if(mode==2){
	vec4 diffuseColor = texture(diffuseTexture, vert.uv);
	vec4 bakedColor = texture(secondTexture,vert.uv);
	vec3 result=((diffuseColor * bakedColor).rgb)*8 + vec3(userDefinedBrightness,userDefinedBrightness,userDefinedBrightness);
	color=vec4(result.rgb,1.0);
	brightColor = vec4(0.0, 0.0, 0.0, 1.0) ;

}
if(mode==1){
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	vec3 r = reflect(-v,n);
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb *(1.0-materialCoefficients.w);
	texColor+=texture(skybox,r).rgb * materialCoefficients.w;
	vec4 result = vec4(texColor * materialCoefficients.x, 1) + vec4(userDefinedBrightness,userDefinedBrightness,userDefinedBrightness,1.0); // ambient
	
	// add directional light contribution
	result.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	result.rgb += phong(n, redPointL.position - vert.position_world, v, redPointL.color * texColor, materialCoefficients.y, redPointL.color, materialCoefficients.z, specularAlpha, true, redPointL.attenuation);
	result.rgb += phong(n, bluePointL.position - vert.position_world, v, bluePointL.color * texColor, materialCoefficients.y, bluePointL.color, materialCoefficients.z, specularAlpha, true, bluePointL.attenuation);
	result.rgb += phong(n, normalPointL.position - vert.position_world, v, normalPointL.color * texColor, materialCoefficients.y, normalPointL.color, materialCoefficients.z, specularAlpha, true, normalPointL.attenuation);

	float brightness = dot(result.rgb, vec3(1, 0, 1));  
    if(brightness > 0.5){
        brightColor = vec4(result.rgb,1.0);}
    else {
        brightColor = vec4(0.0, 0.0, 0.0, 1.0);}
		color=vec4(result.rgb,1.0);
}
if(mode==0){
vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec4 texColor = texture(diffuseTexture, vert.uv).rgba;
	color = texColor; // ambient
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, vec3(1,1,1), dirL.color * texColor.rgb, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0)) + userDefinedBrightness ;
			
}
}

